@@ -3,7 +3,7 @@ import json
 import pickle
 
 from functools import wraps
-from typing import Callable
+from typing import Callable, Dict, Generic, TypeVar, Union, cast
 
 import requests
 
@@ -19,6 +19,20 @@ CONFIG_REQUEST_TIMEOUT = "ckanext.drupal_api.timeout"
 DEFAULT_REQUEST_TIMEOUT = 5
 DEFAULT_CACHE_DURATION = 3600
 
+T = TypeVar('T')
+
+class DontCache(Generic[T]):
+    __slots__ = ('value',)
+    value: T
+
+    def __init__(self, value: T):
+        self.value = value
+
+    def unwrap(self) -> T:
+        return self.value
+
+MaybeNotCached = Union[T, DontCache[T]]
+
 
 class Drupal:
     url: str
@@ -34,7 +48,7 @@ class Drupal:
     def __init__(self, url: str):
         self.url = url.strip("/")
 
-    def _request(self, entity_type: str, entity_name: str):
+    def _request(self, entity_type: str, entity_name: str)->Dict:
         url = self.url + f"/jsonapi/{entity_type}/{entity_name}"
         timeout = tk.asint(
             tk.config.get(CONFIG_REQUEST_TIMEOUT, DEFAULT_REQUEST_TIMEOUT)
@@ -50,7 +64,7 @@ class Drupal:
         return self._request("menu_items", name)
 
 
-def cached(func: Callable) -> Callable:
+def cached(func: Callable[..., MaybeNotCached[T]]) -> Callable[..., T]:
     @wraps(func)
     def wrapper(*args, **kwargs):
         conn = redis.connect_to_redis()
@@ -58,12 +72,14 @@ def cached(func: Callable) -> Callable:
         key = pickle.dumps((args, kwargs))
         value = conn.get(key)
         if value:
-            return json.loads(value)
+            return cast(T, json.loads(value))
 
         value = func(*args, **kwargs)
         cache_duration = tk.asint(
             tk.config.get(CONFIG_CACHE_DURATION, DEFAULT_CACHE_DURATION)
         )
+        if isinstance(value, DontCache):
+            value = cast(T, value.unwrap())
         conn.set(key, json.dumps(value), ex=cache_duration)
         return value
 
